/***************************************************************************/
/* libSIMD Mathematics Library using SIMD code.                            */
/* Copyright (C) 2000,2001 by Iain Nicholson <iain@delphinus.demon.co.uk>  */
/* Last modified :   7th February 2002                                     */
/*                                                                         */
/* This file is part of libSIMD.                                           */
/*                                                                         */
/*  libSIMD is free software; you can redistribute it and/or modify it     */
/*  under the terms of the GNU Lesser General Public License as published  */
/*  by the Free Software Foundation; either version 2 of the License, or   */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  libSIMD is distributed in the hope that it will be useful, but WITHOUT */
/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public   */
/*  License for more details.                                              */
/*                                                                         */
/*  You should have received a copy of the GNU Lesser General Public       */
/*  License along with libSIMD; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307   */
/*  USA                                                                    */
/***************************************************************************/

#ifndef LIBSIMD_MATRIXD
 #include "matrixd.h"
#endif

void simd_matd_2x2_mul(double a[2][2], double b[2][2], double c[2][2])
{


	/* this is what it does in C : */
	/* (loops unolled by hand) ;-> */

	c[0][0]=a[0][0]*b[0][0]+a[0][1]*b[1][0]; c[0][1]=a[0][0]*b[0][1]+a[0][1]*b[1][1];
	c[1][0]=a[1][0]*b[0][0]+a[1][1]*b[1][0]; c[1][1]=a[1][0]*b[0][1]+a[1][1]*b[1][1];

}

void simd_matd_2x2_det(double a[2][2], double *b)
{


	/* this is what it does in C : */

	*b=(a[0][0]*a[1][1])-(a[1][0]*a[0][1]);



}


void simd_matd_2x1_mul(double a[2][2], double b[2], double c[2])
{

	/* this is what it does in C : */

	c[0]=(a[0][0]*b[0])+(a[0][1]*b[1]);
	c[1]=(a[1][0]*b[0])+(a[1][1]*b[1]);


}

void simd_matd_3x3_mul(double a[3][3], double b[3][3], double c[3][3])
{


	/* this is what it does in C : */
	/* I may decide to implement this as a couple of loops */

	c[0][0]=a[0][0]*b[0][0]+a[0][1]*b[1][0]+a[0][2]*b[2][0];
	c[0][1]=a[0][0]*b[0][1]+a[0][1]*b[1][1]+a[0][2]*b[2][1];
	c[0][2]=a[0][0]*b[0][2]+a[0][1]*b[1][2]+a[0][2]*b[2][2];
	
	c[1][0]=a[1][0]*b[0][0]+a[1][1]*b[1][0]+a[1][2]*b[2][0];
	c[1][1]=a[1][0]*b[0][1]+a[1][1]*b[1][1]+a[1][2]*b[2][1];
	c[1][2]=a[1][0]*b[0][2]+a[1][1]*b[1][2]+a[1][2]*b[2][2];
	
	c[2][0]=a[2][0]*b[0][0]+a[2][1]*b[1][0]+a[2][2]*b[2][0];
	c[2][1]=a[2][0]*b[0][1]+a[2][1]*b[1][1]+a[2][2]*b[2][1];
	c[2][2]=a[2][0]*b[0][2]+a[2][1]*b[1][2]+a[2][2]*b[2][2];
	

}

void simd_matd_3x3_det(double a[3][3], double *b)
{


	/* this is what it does in C : */

	*b = (a[0][0]* (a[1][1]*a[2][2] - a[2][1]*a[1][2]))
	    -(a[0][1]* (a[1][0]*a[2][2] - a[2][0]*a[1][2]))
	    +(a[0][2]* (a[1][0]*a[2][1] - a[2][0]*a[1][1]));
	

}

void simd_matd_3x1_mul(double a[3][3], double b[3], double c[3])
{


	/* this is what it does in C : */

	c[0]=(a[0][0]*b[0])+(a[0][1]*b[1])+(a[0][2]*b[2]);
	c[1]=(a[1][0]*b[0])+(a[1][1]*b[1])+(a[1][2]*b[2]);
	c[2]=(a[2][0]*b[0])+(a[2][1]*b[1])+(a[2][2]*b[2]);


}

void simd_matd_4x4_mul(double a[4][4], double b[4][4], double c[4][4])
{


	/* this is what it does in C : */
	/* I may decide to implement this as a couple of loops */

	c[0][0]=a[0][0]*b[0][0]+a[0][1]*b[1][0]+a[0][2]*b[2][0]+a[0][3]*b[3][0];
	c[0][1]=a[0][0]*b[0][1]+a[0][1]*b[1][1]+a[0][2]*b[2][1]+a[0][3]*b[3][1];
	c[0][2]=a[0][0]*b[0][2]+a[0][1]*b[1][2]+a[0][2]*b[2][2]+a[0][3]*b[3][2];
	c[0][3]=a[0][0]*b[0][3]+a[0][1]*b[1][3]+a[0][2]*b[2][3]+a[0][3]*b[3][3];
	c[1][0]=a[1][0]*b[0][0]+a[1][1]*b[1][0]+a[1][2]*b[2][0]+a[1][3]*b[3][0];
	c[1][1]=a[1][0]*b[0][1]+a[1][1]*b[1][1]+a[1][2]*b[2][1]+a[1][3]*b[3][1];
	c[1][2]=a[1][0]*b[0][2]+a[1][1]*b[1][2]+a[1][2]*b[2][2]+a[1][3]*b[3][2];
	c[1][3]=a[1][0]*b[0][3]+a[1][1]*b[1][3]+a[1][2]*b[2][3]+a[1][3]*b[3][3];
	c[2][0]=a[2][0]*b[0][0]+a[2][1]*b[1][0]+a[2][2]*b[2][0]+a[2][3]*b[3][0];
	c[2][1]=a[2][0]*b[0][1]+a[2][1]*b[1][1]+a[2][2]*b[2][1]+a[2][3]*b[3][1];
	c[2][2]=a[2][0]*b[0][2]+a[2][1]*b[1][2]+a[2][2]*b[2][2]+a[2][3]*b[3][2];
	c[2][3]=a[2][0]*b[0][3]+a[2][1]*b[1][3]+a[2][2]*b[2][3]+a[2][3]*b[3][3];
	c[3][0]=a[3][0]*b[0][0]+a[3][1]*b[1][0]+a[3][2]*b[2][0]+a[3][3]*b[3][0];
	c[3][1]=a[3][0]*b[0][1]+a[3][1]*b[1][1]+a[3][2]*b[2][1]+a[3][3]*b[3][1];
	c[3][2]=a[3][0]*b[0][2]+a[3][1]*b[1][2]+a[3][2]*b[2][2]+a[3][3]*b[3][2];
	c[3][3]=a[3][0]*b[0][3]+a[3][1]*b[1][3]+a[3][2]*b[2][3]+a[3][3]*b[3][3];

}

void simd_matd_4x1_mul(double a[4][4], double b[4], double c[4])
{

	/* this is what it does in C : */

	c[0]=(a[0][0]*b[0])+(a[0][1]*b[1])+(a[0][2]*b[2])+(a[0][3]*b[3]);
	c[1]=(a[1][0]*b[0])+(a[1][1]*b[1])+(a[1][2]*b[2])+(a[1][3]*b[3]);
	c[2]=(a[2][0]*b[0])+(a[2][1]*b[1])+(a[2][2]*b[2])+(a[2][3]*b[3]);
	c[3]=(a[3][0]*b[0])+(a[3][1]*b[1])+(a[3][2]*b[2])+(a[3][3]*b[3]);



}