/***************************************************************************/
/* libSIMD Mathematics Library using SIMD code.                            */
/* Copyright (C) 2000,2001 by Iain Nicholson <iain@delphinus.demon.co.uk>  */
/* Last modified : 14th February 2002                                      */
/*                                                                         */
/* This file is part of libSIMD.                                           */
/*                                                                         */
/*  libSIMD is free software; you can redistribute it and/or modify it     */
/*  under the terms of the GNU Lesser General Public License as published  */
/*  by the Free Software Foundation; either version 2 of the License, or   */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  libSIMD is distributed in the hope that it will be useful, but WITHOUT */
/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public   */
/*  License for more details.                                              */
/*                                                                         */
/*  You should have received a copy of the GNU Lesser General Public       */
/*  License along with libSIMD; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307   */
/*  USA                                                                    */
/***************************************************************************/

#ifndef LIBSIMD_VECTORD
 #include "vectord.h"
#endif

/***************************************************************************/
/* 10th December 2001: Now we really tidy this up aiming for Version 0.0.1 */
/***************************************************************************/

/*************/
/* 2-vectors */
/*************/

void simd_vecd2_assign(double a, double b, double *c)
{

	/* this is what it does in C : */
	c[0]=a;
	c[1]=b;

}

void simd_vecd2_copy(double *a, double *b)
{


	/* this is what it does in C : */
	b[0]=a[0];
	b[1]=a[1];

}

void simd_vecd2_add(double *a, double *b, double *c)
{


	/* this is what it does in C : */
	c[0]=a[0]+b[0];
	c[1]=a[1]+b[1];

}

void simd_vecd2_sub(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]-b[0];
	c[1]=a[1]-b[1];

}

void simd_vecd2_scalar_product(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	*c=(a[0]*b[0])+(a[1]*b[1]);

}


/*void simd_vecd2_vecdtor_product(double *, double *, double *, double *, double *, double *);*/

void simd_vecd2_mag(double *a, double *b)
{
	/* this is what it does in C : */
	*b=sqrt(a[0]*a[0]+a[1]*a[1]);
}

void simd_vecd2_mul(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]*b[0];
	c[1]=a[1]*b[1];

}

void simd_vecd2_div(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]/b[0];
	c[1]=a[1]/b[1];

}

void simd_vecd2_recip(double *a, double *b)
{

	/* this is what it does in C : */
	b[0]=1/a[0];
	b[1]=1/a[1];

}

/*void simd_vecd2_neg();*/

/*************/
/* 3-vectors */
/*************/

/***********************************************************************/
/* Assign doubleing point numbers a, b and c to vector pointed to by v */
/* using SIMD (of any benefit?)                                        */
/***********************************************************************/

void simd_vecd3_assign(double a, double b, double c, double *v)
{

	/* in C : */
	v[0]=a;
	v[1]=b;
	v[2]=c;

} 

/*****************************************/
/* Copy vector a into vetor b using SIMD */
/*****************************************/

void simd_vecd3_copy(double *a, double *b)
{

	/* in C : */
	b[0]=a[0];
	b[1]=a[1];
	b[2]=a[2];

}

/********************************************************/
/* Sum two 3-element doubleing point vectors using SIMD */
/* *a , *b : pointers to vectors to be added            */
/* *c      : pointer to vector in which to store result */
/********************************************************/

void simd_vecd3_add(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]+b[0];
	c[1]=a[1]+b[1];
	c[2]=a[2]+b[2];

}


/*************************************************************/
/* Subtract two 3-element doubleing point vectors using SIMD */
/* *a , *b : pointers to vectors to be subtracted            */
/* *c      : pointer to vector in which to store result      */
/*************************************************************/

void simd_vecd3_sub(double *a, double *b, double *c)
{


	/* this is what it does in C : */
	c[0]=a[0]-b[0];
	c[1]=a[1]-b[1];
	c[2]=a[2]-b[2];

}


/**********************************************/
/* perform two scalar products on 2 3-vectors */
/**********************************************/

void simd_vecd3_scalar_product(double *a, double *b, double *c)
{

	*c=(a[0]*b[0])+(a[1]*b[1])+(a[2]*b[2]);

}

void simd_vecd3_mag(double *a, double *b)
{
	/* this is what it does in C : */
	*b=sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
}


/***********************************************************************/
/* Multiply six doubleing-point numbers using SIMD                     */
/* *a , *b : pointers to arrays of 4 doubles containing the input data */
/* *c      : pointer to array of 4 doubles in which to store result    */
/***********************************************************************/

void simd_vecd3_mul(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]*b[0];
	c[1]=a[1]*b[1];
	c[2]=a[2]*b[2];

}

void simd_vecd3_recip(double *a, double *b)
{


	/* this is what it does in C : */
	b[0]=1/a[0];
	b[1]=1/a[1];
	b[2]=1/a[2];

}

/***********************************************************************/
/* Divide three doubleing-point numbers using SIMD                     */
/* *a , *b : pointers to arrays of 4 doubles containing the input data */
/* *c      : pointer to array of 4 doubles in which to store result    */
/***********************************************************************/

void simd_vecd3_div(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]/b[0];
	c[1]=a[1]/b[1];
	c[2]=a[2]/b[2];


}

/*************************************************************************/
/* Assign doubleing point numbers a,b ,c and d to vector pointed to by v */
/* using SIMD (of any benefit?)                                          */
/*************************************************************************/

void simd_vecd4_assign(double a, double b, double c, double d, double *v)
{


	/* in C : */
	v[0]=a;
	v[1]=b;
	v[2]=c;
	v[3]=d;

} 

/*****************************************/
/* Copy vector a into vetor b using SIMD */
/*****************************************/

void simd_vecd4_copy(double *a, double *b)
{

	/* in C : */
	b[0]=a[0];
	b[1]=a[1];
	b[2]=a[2];
	b[3]=a[3];

}

/********************************************************/
/* Sum two 4-element doubleing point vectors using SIMD */
/* *a , *b : pointers to vectors to be added            */
/* *c      : pointer to vector in which to store result */
/********************************************************/

void simd_vecd4_add(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]+b[0];
	c[1]=a[1]+b[1];
	c[2]=a[2]+b[2];
	c[3]=a[3]+b[3];

}


/*************************************************************/
/* Subtract two 4-element doubleing point vectors using SIMD */
/* *a , *b : pointers to vectors to be subtracted            */
/* *c      : pointer to vector in which to store result      */
/*************************************************************/

void simd_vecd4_sub(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]-b[0];
	c[1]=a[1]-b[1];
	c[2]=a[2]-b[2];
	c[3]=a[3]-b[3];

}


/***********************************/
/* scalar product c=a.b, 4-vectors */
/***********************************/

void simd_vecd4_scalar_product(double *a, double *b, double *c)
{

	c[0]=(a[0]*b[0])+(a[1]*b[1])+(a[2]*b[2])+(a[3]*b[3]);
	
}

void simd_vecd4_mag(double *a, double *b)
{
	/* this is what it does in C : */
	*b=sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]+a[3]*a[3]);
}

/***********************************************************************/
/* Multiply eight doubleing-point numbers using SIMD                   */
/* *a , *b : pointers to arrays of 4 doubles containing the input data */
/* *c      : pointer to array of 4 doubles in which to store result    */
/***********************************************************************/

void simd_vecd4_mul(double *a, double *b, double *c)
{


	/* this is what it does in C : */
	c[0]=a[0]*b[0];
	c[1]=a[1]*b[1];
	c[2]=a[2]*b[2];
	c[3]=a[3]*b[3];

}

void simd_vecd4_recip(double *a, double *b)
{

	/* this is what it does in C : */
	b[0]=1/a[0];
	b[1]=1/a[1];
	b[2]=1/a[2];
	b[3]=1/a[3];

}

/***********************************************************************/
/* Divide four doubleing-point numbers using SIMD                      */
/* *a , *b : pointers to arrays of 4 doubles containing the input data */
/* *c      : pointer to array of 4 doubles in which to store result    */
/***********************************************************************/

void simd_vecd4_div(double *a, double *b, double *c)
{

	/* this is what it does in C : */
	c[0]=a[0]/b[0];
	c[1]=a[1]/b[1];
	c[2]=a[2]/b[2];
	c[3]=a[3]/b[3];

}
