/***************************************************************************/
/* Test program for libSIMD, the Mathematics Library using SIMD code.      */
/* Copyright (C) 2000-2002 by Iain Nicholson  <iain@delphinus.demon.co.uk> */
/* Last modified : 14th February 2002                                      */
/*                                                                         */
/* This file is part of libSIMD.                                           */
/*                                                                         */
/*  libSIMD is free software; you can redistribute it and/or modify it     */
/*  under the terms of the GNU Lesser General Public License as published  */
/*  by the Free Software Foundation; either version 2 of the License, or   */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  libSIMD is distributed in the hope that it will be useful, but WITHOUT */
/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public   */
/*  License for more details.                                              */
/*                                                                         */
/*  You should have received a copy of the GNU Lesser General Public       */
/*  License along with libSIMD; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307   */
/*  USA                                                                    */
/***************************************************************************/

#include <math.h>
#include <stdio.h>
#include "simds.h"

#define PI 3.14159269

/* this will do for now, and is merely to show that things work */

void test_scalars(void);
void test_trigs(void);
void test_vectors(void);
void test_vectors2(void);
void test_vectors3(void);
void test_vectors4(void);
void test_matrixs(void);
void test_matrixs2(void);
void test_matrixs3(void);
void test_matrixs4(void);
void test_complexs(void);
void test_dsps(void);
void test_quats(void);

void mats_show_2x2(float [2][2]);
void mats_show_3x3(float [3][3]);
void mats_show_4x4(float [4][4]);

int main(void)
{

	printf("libSIMD single-precision tests.\n");

	test_scalars();
	test_trigs();
	test_vectors();
	test_matrixs();
	test_complexs();
	test_dsps();
	test_quats();

	printf("\n");

	
	return 0;
}

/*
**  End of main()
*/

void test_scalars()
{

	printf("Scalar functions.\n");
	printf("None implemented.\n");
	printf("Done.\n");
	
}

void test_trigs()
{

	float angle, result;
	float k;
	int i;

	printf("Trigonometry functions.\n");
	k=PI/180;
	for(i=0; i<360; i+=30)
	{
		angle=(float)i*k;
		femms();
		simd_sins(&angle, &result);
		femms();
		printf("simd_sins(%d)=%f\n",i,(double)result);
	}

	for(i=0; i<360; i+=30)
	{
		angle=(float)i*k;
		femms();
		simd_coss(&angle, &result);
		femms();
		printf("simd_coss(%d)=%f\n",i,(double)result);
	}

	for(i=0; i<10; i++)
	{
		angle=(float)i;
		femms();
		simd_sinhs(&angle, &result);
		femms();
		printf("simd_sinhs(%d)=%f\n",i,(double)result);
	}

	for(i=0; i<10; i++)
	{
		angle=(float)i;
		femms();
		simd_coshs(&angle, &result);
		femms();
		printf("simd_coshs(%d)=%f\n",i,(double)result);
	}

	for(i=0; i<10; i++)
	{
		angle=(float)i;
		femms();
		simd_exps(&angle, &result);
		femms();
		printf("simd_expss(%d)=%f\n",i,(double)result);
	}

	printf("Done.\n");

}

void test_vectors()
{

	printf("Vector functions.\n");

	test_vectors2();
	test_vectors3();
	test_vectors4();
		
	printf("Done.\n");
	
}


void test_vectors2(void)
{

	float a[2], b[2], c[2];
	float x, y;

	
	printf("2-vector functions.\n");
	printf("\n");
	
	printf("assign:\n");
	x=1.0;
	y=2.0;
	printf("x=%f, y=%f\n", (double) x, (double) y);
	femms();
	simd_vecs2_assign(x, y, a);
	femms();
	printf("a[0]=%f, a[1]=%f\n\n", (double) a[0], (double) a[1]);

	printf("copy:\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	femms();
	simd_vecs2_copy(a, b);
	femms();
	printf("b[0]=%f, b[1]=%f\n\n", (double) b[0], (double) b[1]);
	
	printf("add\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	b[0]=3.0;
	b[1]=4.0;
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_add(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f\n\n", (double) c[0], (double) c[1]);

	printf("sub\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_sub(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f\n\n", (double) c[0], (double) c[1]);


	printf("scalar product\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_scalar_product(a, b, &x);
	femms();
	printf("x=%f\n\n", (double) x);

	printf("magnitude\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	femms();
	simd_vecs2_mag(a, &x);
	femms();
	printf("x=%f\n\n", (double) x);

	printf("multiply elements\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_mul(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f\n\n", (double) c[0], (double) c[1]);

	printf("divide elements\n");
	printf("a[0]=%f, a[1]=%f\n", (double) a[0], (double) a[1]);
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_div(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f\n\n", (double) c[0], (double) c[1]);

	printf("reciprocal of elements\n");
	printf("b[0]=%f, b[1]=%f\n", (double) b[0], (double) b[1]);
	femms();
	simd_vecs2_recip(b, c);
	femms();
	printf("c[0]=%f, c[1]=%f\n\n", (double) c[0], (double) c[1]);	
	
	printf("Done.\n");
	
}

void test_vectors3(void)
{
	float a[3], b[3], c[3];
	float x, y, z;
	
	printf("3-vector functions.\n");
	printf("\n");
	
	printf("assign:\n");
	x=1.0;
	y=2.0;
	z=3.0;
	printf("x=%f, y=%f, z=%f\n", (double) x, (double) y, (double) z);
	femms();
	simd_vecs3_assign(x, y, z, a);
	femms();
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);

	printf("copy:\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n", (double) a[0], (double) a[1], (double) a[2]);
	femms();
	simd_vecs3_copy(a, b);
	femms();
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	
	printf("add\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	b[0]=4.0;
	b[1]=5.0;
	b[2]=6.0;
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_add(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f\n\n", (double) c[0], (double) c[1], (double) c[2]);

	printf("sub\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_sub(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f\n\n", (double) c[0], (double) c[1], (double) c[2]);


	printf("scalar product\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_scalar_product(a, b, &x);
	femms();
	printf("x=%f\n\n", (double) x);

	printf("magnitude\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	femms();
	simd_vecs3_mag(a, &x);
	femms();
	printf("x=%f\n\n", (double) x);

	printf("multiply elements\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_mul(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f\n\n", (double) c[0], (double) c[1], (double) c[2]);

	printf("divide elements\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f\n\n", (double) a[0], (double) a[1], (double) a[2]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_div(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f\n\n", (double) c[0], (double) c[1], (double) c[2]);

	printf("reciprocal of elements\n");
	printf("b[0]=%f, b[1]=%f, b[2]=%f\n\n", (double) b[0], (double) b[1], (double) b[2]);
	femms();
	simd_vecs3_recip(b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f\n\n", (double) c[0], (double) c[1], (double) c[2]);

	printf("Done.\n");
	
}

void test_vectors4(void)
{
	float a[4], b[4], c[4];
	float x1, x2, x3, x4;
	
	printf("4-vector functions.\n");
	printf("\n");
	
	printf("assign:\n");
	x1=1.0;
	x2=2.0;
	x3=3.0;
	x4=4.0;
	printf("x1=%f, x2=%f, x3=%f, x4=%f\n", (double) x1, (double) x2, (double) x3, (double) x4);
	femms();
	simd_vecs4_assign(x1, x2, x3, x4, a);
	femms();
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);

	printf("copy:\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	femms();
	simd_vecs4_copy(a, b);
	femms();
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	
	printf("add\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	b[0]=4.0;
	b[1]=5.0;
	b[2]=6.0;
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_add(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f, c[3]=%f\n\n", (double) c[0], (double) c[1], (double) c[2], (double) c[3]);

	printf("sub\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_sub(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f, c[3]=%f\n\n", (double) c[0], (double) c[1], (double) c[2], (double) c[3]);


	printf("scalar product\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_scalar_product(a, b, &x1);
	femms();
	printf("x1=%f\n\n", (double) x1);

	printf("magnitude\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	femms();
	simd_vecs4_mag(a, &x1);
	femms();
	printf("x1=%f\n\n", (double) x1);

	printf("multiply elements\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_mul(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f, c[3]=%f\n\n", (double) c[0], (double) c[1], (double) c[2], (double) c[3]);

	printf("divide elements\n");
	printf("a[0]=%f, a[1]=%f, a[2]=%f, a[3]=%f\n\n", (double) a[0], (double) a[1], (double) a[2], (double) a[3]);
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_div(a, b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f, c[3]=%f\n\n", (double) c[0], (double) c[1], (double) c[2], (double) c[3]);

	printf("reciprocal of elements\n");
	printf("b[0]=%f, b[1]=%f, b[2]=%f, b[3]=%f\n\n", (double) b[0], (double) b[1], (double) b[2], (double) b[3]);
	femms();
	simd_vecs4_recip(b, c);
	femms();
	printf("c[0]=%f, c[1]=%f, c[2]=%f, c[3]=%f\n\n", (double) c[0], (double) c[1], (double) c[2], (double) c[3]);

	printf("Done.\n");
	
}

void test_matrixs(void)
{

	printf("Matrix functions.\n");

	test_matrixs2();
	test_matrixs3();
	test_matrixs4();
	 
	printf("Done.\n");
	
}


void test_matrixs2(void)
{
	float a[2][2], b[2][2], c[2][2];
	float p[2], q[2];
	float x;
	int i, j;

	printf("2x2 Matrix functions.\n");
	printf("\n");

	printf("multiply 2x2 x 2x2:\n");
	for(j=0; j<2; j++)
	{
		for(i=0; i<2; i++)
		{
			a[j][i]=(float)((j+1)*(i+1));
		}
	}
	for(j=0; j<2; j++)
	{
		for(i=0; i<2; i++)
		{
			b[j][i]=(float)((j+1)*(i+1));
		}
	}
	printf("a:\n");
	mats_show_2x2(a);
	printf("b:\n");
	mats_show_2x2(b);
	femms();
	simd_mats_2x2_mul(a, b, c);
	femms();
	printf("c:\n");
	mats_show_2x2(c);
	printf("\n");

	printf("determinant:\n");
	printf("a:\n");
	mats_show_2x2(a);
	simd_mats_2x2_det(a, &x);
	printf("x=%f\n\n", (double) x);

	printf("multiply 2x2 x 2x1:\n");
	p[0]=2.0;
	p[1]=4.0;
	printf("a:\n");
	mats_show_2x2(a);
	printf("p:\n");	
	printf("p[0]=%f, p[1]=%f\n", (double) p[0], (double) p[1]);
	femms();
	simd_mats_2x1_mul(a, p, q);
	femms();
	printf("q:\n");	
	printf("q[0]=%f, q[1]=%f\n", (double) q[0], (double) q[1]);
	
	printf("Done.\n");
	
}

void test_matrixs3(void)
{
	float a[3][3], b[3][3], c[3][3];
	float p[3], q[3];
	float x;
	int i, j;
	
	printf("3x3 Matrix functions.\n");
	printf("\n");

	printf("multiply 3x3 x 3x3:\n");
	for(j=0; j<3; j++)
	{
		for(i=0; i<3; i++)
		{
			a[j][i]=(float)((j+1)*(i+1));
		}
	}
	for(j=0; j<3; j++)
	{
		for(i=0; i<3; i++)
		{
			b[j][i]=(float)((j+1)*(i+1));
		}
	}
	printf("a:\n");
	mats_show_3x3(a);
	printf("b:\n");
	mats_show_3x3(b);
	femms();
	simd_mats_3x3_mul(a, b, c);
	femms();
	printf("c:\n");
	mats_show_3x3(c);
	printf("\n");

	printf("determinant:\n");
	printf("a:\n");
	mats_show_3x3(a);
	simd_mats_3x3_det(a, &x);
	printf("x=%f\n\n", (double) x);

	printf("multiply 3x3 x 3x1:\n");
	p[0]=2.0;
	p[1]=4.0;
	p[2]=6.0;
	printf("a:\n");
	mats_show_3x3(a);
	printf("p:\n");	
	printf("p[0]=%f, p[1]=%f, p[2]=%f\n", (double) p[0], (double) p[1], (double) p[2]);
	femms();
	simd_mats_3x1_mul(a, p, q);
	femms();
	printf("q:\n");	
	printf("q[0]=%f, q[1]=%f, q[2]=%f\n", (double) q[0], (double) q[1], (double) q[2]);
			
	printf("Done.\n");
	
}

void test_matrixs4(void)
{
	float a[4][4], b[4][4], c[4][4];
	float p[4], q[4];
	float x;
	int i, j;
	printf("4x4 Matrix functions.\n");
	printf("\n");

	printf("multiply 4x4 x 4x4:\n");
	for(j=0; j<4; j++)
	{
		for(i=0; i<4; i++)
		{
			a[j][i]=(float)((j+1)*(i+1));
		}
	}
	for(j=0; j<4; j++)
	{
		for(i=0; i<4; i++)
		{
			b[j][i]=(float)((j+1)*(i+1));
		}
	}
	printf("a:\n");
	mats_show_4x4(a);
	printf("b:\n");
	mats_show_4x4(b);
	femms();
	simd_mats_4x4_mul(a, b, c);
	femms();
	printf("c:\n");
	mats_show_4x4(c);
	printf("\n");

	printf("multiply 4x4 x 4x1:\n");
	p[0]=2.0;
	p[1]=4.0;
	p[2]=6.0;
	p[3]=8.0;
	printf("a:\n");
	mats_show_4x4(a);
	printf("p:\n");	
	printf("p[0]=%f, p[1]=%f, p[2]=%f, p[3]=%f\n", (double) p[0], (double) p[1], (double) p[2], (double) p[3]);
	femms();
	simd_mats_4x1_mul(a, p, q);
	femms();
	printf("q:\n");	
	printf("q[0]=%f, q[1]=%f, q[2]=%f, q[3]=%f\n", (double) q[0], (double) q[1], (double) q[2], (double) q[3]);

	printf("Done.\n");
	
}

void test_complexs(void)
{

	printf("Complex functions.\n");
	printf("Done.\n");
	
}

void test_dsps(void)
{

	printf("DSP functions.\n");
	printf("None implemented.\n");
	printf("Done.\n");
	
}

void test_quats(void)
{

	printf("Quaternion functions.\n");
	printf("None implemented.\n");
	printf("Done.\n");
	
}

void mats_show_2x2(float a[2][2])
{
	int i, j;
	for(j=0; j<2; j++)
	{
		for(i=0; i<2; i++)
		{
			printf("%f ",a[j][i]);
		}
		printf("\n");
	}
}

void mats_show_3x3(float a[3][3])
{
	int i, j;
	for(j=0; j<3; j++)
	{
		for(i=0; i<3; i++)
		{
			printf("%f ",a[j][i]);
		}
		printf("\n");
	}
}

void mats_show_4x4(float a[4][4])
{
	int i, j;
	for(j=0; j<4; j++)
	{
		for(i=0; i<4; i++)
		{
			printf("%f ",a[j][i]);
		}
		printf("\n");
	}
}