/***************************************************************************/
/* libSIMD Mathematics Library using SIMD code.                            */
/* Copyright (C) 2000,2001 by Iain Nicholson <iain@delphinus.demon.co.uk>  */
/* Last modified : 18th November 2001                                      */
/*                                                                         */
/* This file is part of libSIMD.                                           */
/*                                                                         */
/*  libSIMD is free software; you can redistribute it and/or modify it     */
/*  under the terms of the GNU Lesser General Public License as published  */
/*  by the Free Software Foundation; either version 2 of the License, or   */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  libSIMD is distributed in the hope that it will be useful, but WITHOUT */
/*  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public   */
/*  License for more details.                                              */
/*                                                                         */
/*  You should have received a copy of the GNU Lesser General Public       */
/*  License along with libSIMD; if not, write to the Free Software         */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307   */
/*  USA                                                                    */
/***************************************************************************/

#ifndef LIBSIMD_MISC
 #include "misc.h"
#endif




void vec_show(float *v)
{
	int i;
	for(i=0; i<4; i++) printf("%f ",v[i]);
	printf("\n");
}

void vec_show3(float *v)
{
	int i;
	for(i=0; i<3; i++) printf("%f ",v[i]);
	printf("\n");
}

void unpack(float *a, float *b, float *c)
{
#ifdef USE_3DNOW
		
	/* set up pointers : %eax = a, %ebx = b, %ecx = c */
	/* load up mmx registers */
	/* perform the unpack */
	/* store the results */
	
	__asm__ __volatile__(

    "movq  (%%eax), %%mm0
	 movq  (%%ebx),%%mm1
	 movq  (%%ecx),%%mm2
	 
     punpckldq  %%mm0, %%mm1
     punpckhdq  %%mm0, %%mm2

	 movq  %%mm0,  (%%eax)
     movq  %%mm1,  (%%ebx)
     movq  %%mm2,  (%%ecx)
     "
	 :
	 : "a" (a), "b" (b), "c" (c)
	 : "memory");
#else


#endif
}

void movd(float *a, float *b, float *c)
{
#ifdef USE_3DNOW
		
	/* set up pointers : %eax = a, %ebx = b, %ecx = c */
	/* load up mmx registers */
	/* perform the movs */
	/* store the results */
	
	__asm__ __volatile__(

    "movq  (%%eax), %%mm0
	 movd  (%%ebx),%%mm0
	 movq  %%mm0,(%%ecx)
     "
	 :
	 : "a" (a), "b" (b), "c" (c)
	 : "memory");
#else


#endif
}
void swap(float *a, float *b)
{
#ifdef USE_3DNOW
		
	/* set up pointers : %eax = a, %ebx = b, %ecx = c */
	/* load up mmx registers */
	/* perform the unpack */
	/* store the results */
	
	__asm__ __volatile__(

    "movq  (%%eax), %%mm0
	 movq  (%%ebx),%%mm1
	 
     punpckldq  %%mm0, %%mm1
     punpckhdq  %%mm0, %%mm2

	 movq  %%mm0,  (%%eax)
     movq  %%mm1,  (%%ebx)
     movq  %%mm2,  (%%ecx)
     "
	 :
	 : "a" (a), "b" (b)
	 : "memory");
#else


#endif
}

/* I'm not proud of this and it only works because of
 * side-effects of the way the complier works :-)
 */

void mat_show(float *data, int dim)
{
	int i, j, k;
	k=0;
	for(i=0; i<dim; i++)
	{
		for(j=0; j<dim; j++)
		{
			printf("%f ", data[k++]);
		}
		printf("\n");
	}
}
